import pyarrow as pa
import pyarrow.parquet as pq
from pyspark.sql import SparkSession
from pyspark.sql.types import StructType,StructField, StringType, IntegerType

import csv
import random
import shutil
import sys
import time
import tempfile

chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

random.seed(1)

spark = SparkSession.builder \
      .master("local[*]") \
      .appName("regex_replace_multi_bench") \
      .config("spark.rapids.sql.explain","ALL") \
      .config("spark.rapids.sql.regexp.enabled", "true") \
      .config("spark.rapids.sql.enabled", "false") \
      .config("spark.sql.parquet.columnarReaderBatchSize", "1024") \
      .getOrCreate()  

tmpdir = tempfile.mkdtemp()

def str_gen(str_len, length=4096):
    data = pa.array(["".join(random.choices(chars, k=str_len)) for _ in range(length)])
    table = pa.table([data], names=["a"])
    filename = tmpdir + f"/data_{str_len}.parquet"
    pq.write_table(table, filename)
    return filename

def re_gen(num_choices, choice_length=5):
    return "|".join(["".join(random.choices(chars, k=choice_length)) for _ in range(num_choices)])


# get rid of spark context initialization overhead

try:
    df = spark.createDataFrame([["b"]], ["a"])
    df.selectExpr("a").collect()

    str_lens = [10, 25] + [50*2**i for i in range(1,6)] + list(range(2000, 31000, 1000))

    data = []

    for slen in str_lens:
        filename = str_gen(slen)
        regex = re_gen(2)
        df = spark.read.parquet(filename)

        rdf = df.selectExpr(
            f"regexp_replace(a, '{regex}', '{'*' * 5}')"
        )
        s = time.time()
        rdf.write.parquet(tmpdir + "/result.parquet", "overwrite")
        t = time.time() - s
        data.append({"String Length": slen, "Execution Time": t})

    with open(sys.argv[1], 'w') as f:
        w = csv.DictWriter(f, ["String Length", "Execution Time"])
        w.writeheader()
        w.writerows(data)

finally:
    shutil.rmtree(tmpdir)